@@ -604,7 +604,7 @@ class V8_EXPORT Object : public Value {
   Local<Context> GetCreationContextChecked();
 
   /** Same as above, but works for Persistents */
-  V8_DEPRECATE_SOON(
+  V8_DEPRECATED(
       "Use MaybeLocal<Context> GetCreationContext(const "
       "PersistentBase<Object>& object)")
   static Local<Context> CreationContext(const PersistentBase<Object>& object);
