@@ -8,7 +8,9 @@
 #include <forward_list>
 #include <memory>
 
+#include "src/ast/ast-value-factory.h"
 #include "src/base/platform/elapsed-timer.h"
+#include "src/base/small-vector.h"
 #include "src/codegen/bailout-reason.h"
 #include "src/common/globals.h"
 #include "src/execution/isolate.h"
@@ -69,7 +71,9 @@ class V8_EXPORT_PRIVATE Compiler : public AllStatic {
 
   static bool Compile(Isolate* isolate, Handle<SharedFunctionInfo> shared,
                       ClearExceptionFlag flag,
-                      IsCompiledScope* is_compiled_scope);
+                      IsCompiledScope* is_compiled_scope,
+                      CreateSourcePositions create_source_positions_flag =
+                          CreateSourcePositions::kNo);
   static bool Compile(Isolate* isolate, Handle<JSFunction> function,
                       ClearExceptionFlag flag,
                       IsCompiledScope* is_compiled_scope);
@@ -104,9 +108,9 @@ class V8_EXPORT_PRIVATE Compiler : public AllStatic {
                      Isolate* isolate);
 
   // Finalize and install code from previously run background compile task.
-  static bool FinalizeBackgroundCompileTask(
-      BackgroundCompileTask* task, Handle<SharedFunctionInfo> shared_info,
-      Isolate* isolate, ClearExceptionFlag flag);
+  static bool FinalizeBackgroundCompileTask(BackgroundCompileTask* task,
+                                            Isolate* isolate,
+                                            ClearExceptionFlag flag);
 
   // Finalize and install optimized code from previously run job.
   static bool FinalizeOptimizedCompilationJob(OptimizedCompilationJob* job,
@@ -495,8 +499,7 @@ using DeferredFinalizationJobDataList =
 class V8_EXPORT_PRIVATE BackgroundCompileTask {
  public:
   // Creates a new task that when run will parse and compile the streamed
-  // script associated with |data| and can be finalized with
-  // Compiler::GetSharedFunctionInfoForStreamedScript.
+  // script associated with |data| and can be finalized with FinalizeScript.
   // Note: does not take ownership of |data|.
   BackgroundCompileTask(ScriptStreamingData* data, Isolate* isolate,
                         v8::ScriptType type);
@@ -504,83 +507,61 @@ class V8_EXPORT_PRIVATE BackgroundCompileTask {
   BackgroundCompileTask& operator=(const BackgroundCompileTask&) = delete;
   ~BackgroundCompileTask();
 
-  // Creates a new task that when run will parse and compile the
-  // |function_literal| and can be finalized with
+  // Creates a new task that when run will parse and compile the top-level
+  // |shared_info| and can be finalized with FinalizeFunction in
   // Compiler::FinalizeBackgroundCompileTask.
   BackgroundCompileTask(
-      const ParseInfo* outer_parse_info, const AstRawString* function_name,
-      const FunctionLiteral* function_literal,
+      Isolate* isolate, Handle<SharedFunctionInfo> shared_info,
+      std::unique_ptr<Utf16CharacterStream> character_stream,
       WorkerThreadRuntimeCallStats* worker_thread_runtime_stats,
       TimedHistogram* timer, int max_stack_size);
 
   void Run();
+  void Run(LocalIsolate* isolate,
+           ReusableUnoptimizedCompileState* reusable_state);
+
+  MaybeHandle<SharedFunctionInfo> FinalizeScript(
+      Isolate* isolate, Handle<String> source,
+      const ScriptDetails& script_details);
+
+  bool FinalizeFunction(Isolate* isolate, Compiler::ClearExceptionFlag flag);
+
+  void AbortFunction();
 
-  ParseInfo* info() {
-    DCHECK_NOT_NULL(info_);
-    return info_.get();
-  }
-  Parser* parser() { return parser_.get(); }
-  UnoptimizedCompilationJobList* compilation_jobs() {
-    return &compilation_jobs_;
-  }
   UnoptimizedCompileFlags flags() const { return flags_; }
-  UnoptimizedCompileState* compile_state() { return &compile_state_; }
-  LanguageMode language_mode() { return language_mode_; }
-  FinalizeUnoptimizedCompilationDataList*
-  finalize_unoptimized_compilation_data() {
-    return &finalize_unoptimized_compilation_data_;
-  }
-
-  int use_count(v8::Isolate::UseCounterFeature feature) const {
-    return use_counts_[static_cast<int>(feature)];
-  }
-  int total_preparse_skipped() const { return total_preparse_skipped_; }
-
-  // Jobs which could not be finalized in the background task, and need to be
-  // finalized on the main thread.
-  DeferredFinalizationJobDataList* jobs_to_retry_finalization_on_main_thread() {
-    return &jobs_to_retry_finalization_on_main_thread_;
-  }
-
-  // Getters for the off-thread finalization results, that create main-thread
-  // handles to the objects.
-  MaybeHandle<SharedFunctionInfo> GetOuterFunctionSfi(Isolate* isolate);
-  Handle<Script> GetScript(Isolate* isolate);
+  LanguageMode language_mode() const { return language_mode_; }
 
  private:
-  // Data needed for parsing, and data needed to to be passed between thread
-  // between parsing and compilation. These need to be initialized before the
-  // compilation starts.
+  void ReportStatistics(Isolate* isolate);
+
+  void ClearFunctionJobPointer();
+
+  // Data needed for parsing and compilation. These need to be initialized
+  // before the compilation starts.
+  Isolate* isolate_for_local_isolate_;
   UnoptimizedCompileFlags flags_;
   UnoptimizedCompileState compile_state_;
-  std::unique_ptr<ParseInfo> info_;
-  std::unique_ptr<Parser> parser_;
-
-  // Data needed for finalizing compilation after background compilation.
-  UnoptimizedCompilationJobList compilation_jobs_;
+  std::unique_ptr<Utf16CharacterStream> character_stream_;
+  int stack_size_;
+  WorkerThreadRuntimeCallStats* worker_thread_runtime_call_stats_;
+  TimedHistogram* timer_;
 
   // Data needed for merging onto the main thread after background finalization.
-  // TODO(leszeks): When these are available, the above fields are not. We
-  // should add some stricter type-safety or DCHECKs to ensure that the user of
-  // the task knows this.
-  Isolate* isolate_for_local_isolate_;
   std::unique_ptr<PersistentHandles> persistent_handles_;
   MaybeHandle<SharedFunctionInfo> outer_function_sfi_;
   Handle<Script> script_;
   IsCompiledScope is_compiled_scope_;
   FinalizeUnoptimizedCompilationDataList finalize_unoptimized_compilation_data_;
   DeferredFinalizationJobDataList jobs_to_retry_finalization_on_main_thread_;
-  int use_counts_[v8::Isolate::kUseCounterFeatureCount] = {0};
+  base::SmallVector<v8::Isolate::UseCounterFeature, 8> use_counts_;
   int total_preparse_skipped_ = 0;
 
   // Single function data for top-level function compilation.
+  MaybeHandle<SharedFunctionInfo> input_shared_info_;
   int start_position_;
   int end_position_;
   int function_literal_id_;
 
-  int stack_size_;
-  WorkerThreadRuntimeCallStats* worker_thread_runtime_call_stats_;
-  TimedHistogram* timer_;
   LanguageMode language_mode_;
 };
 
