@@ -90,6 +90,14 @@ constexpr int64_t TB = static_cast<int64_t>(GB) * 1024;
 #define V8_DEFAULT_STACK_SIZE_KB 984
 #endif
 
+// Helper macros to enable handling of direct C calls in the simulator.
+#if defined(USE_SIMULATOR) && defined(V8_TARGET_ARCH_ARM64)
+#define V8_USE_SIMULATOR_WITH_GENERIC_C_CALLS
+#define V8_IF_USE_SIMULATOR(V) , V
+#else
+#define V8_IF_USE_SIMULATOR(V)
+#endif  // defined(USE_SIMULATOR) && defined(V8_TARGET_ARCH_ARM64)
+
 // Minimum stack size in KB required by compilers.
 constexpr int kStackSpaceRequiredForCompilation = 40;
 
@@ -104,8 +112,15 @@ STATIC_ASSERT(V8_DEFAULT_STACK_SIZE_KB* KB +
                   kStackLimitSlackForDeoptimizationInBytes <=
               MB);
 
-#if defined(V8_SHORT_BUILTIN_CALLS) && \
-    (!defined(V8_COMPRESS_POINTERS) || defined(V8_EXTERNAL_CODE_SPACE))
+// The V8_ENABLE_NEAR_CODE_RANGE_BOOL enables logic that tries to allocate
+// code range within a pc-relative call/jump proximity from embedded builtins.
+// This machinery could help only when we have an opportunity to choose where
+// to allocate code range and could benefit from it. This is the case for the
+// following configurations:
+// - external code space AND pointer compression are enabled,
+// - short builtin calls feature is enabled while pointer compression is not.
+#if (defined(V8_SHORT_BUILTIN_CALLS) && !defined(V8_COMPRESS_POINTERS)) || \
+    defined(V8_EXTERNAL_CODE_SPACE)
 #define V8_ENABLE_NEAR_CODE_RANGE_BOOL true
 #else
 #define V8_ENABLE_NEAR_CODE_RANGE_BOOL false
@@ -115,11 +130,6 @@ STATIC_ASSERT(V8_DEFAULT_STACK_SIZE_KB* KB +
 // physical memory by checking the max old space size.
 const size_t kShortBuiltinCallsOldSpaceSizeThreshold = size_t{2} * GB;
 
-// This constant is used for detecting whether code range could be
-// allocated within the +/- 2GB boundary to builtins' embedded blob
-// to use short builtin calls.
-const size_t kShortBuiltinCallsBoundary = size_t{2} * GB;
-
 // Determine whether dict mode prototypes feature is enabled.
 #ifdef V8_ENABLE_SWISS_NAME_DICTIONARY
 #define V8_ENABLE_SWISS_NAME_DICTIONARY_BOOL true
@@ -866,8 +876,27 @@ inline constexpr bool IsSharedAllocationType(AllocationType kind) {
          kind == AllocationType::kSharedMap;
 }
 
-// TODO(ishell): review and rename kWordAligned to kTaggedAligned.
-enum AllocationAlignment { kWordAligned, kDoubleAligned, kDoubleUnaligned };
+enum AllocationAlignment {
+  // The allocated address is kTaggedSize aligned (this is default for most of
+  // the allocations).
+  kTaggedAligned,
+  // The allocated address is kDoubleSize aligned.
+  kDoubleAligned,
+  // The (allocated address + kTaggedSize) is kDoubleSize aligned.
+  kDoubleUnaligned
+};
+
+#ifdef V8_HOST_ARCH_32_BIT
+#define USE_ALLOCATION_ALIGNMENT_BOOL true
+#else
+#ifdef V8_COMPRESS_POINTERS
+// TODO(ishell, v8:8875): Consider using aligned allocations once the
+// allocation alignment inconsistency is fixed. For now we keep using
+// unaligned access since both x64 and arm64 architectures (where pointer
+// compression is supported) allow unaligned access to doubles and full words.
+#endif  // V8_COMPRESS_POINTERS
+#define USE_ALLOCATION_ALIGNMENT_BOOL false
+#endif  // V8_HOST_ARCH_32_BIT
 
 enum class AccessMode { ATOMIC, NON_ATOMIC };
 
@@ -938,7 +967,7 @@ enum ParseRestriction : bool {
 };
 
 // State for inline cache call sites. Aliased as IC::State.
-enum InlineCacheState {
+enum class InlineCacheState {
   // No feedback will be collected.
   NO_FEEDBACK,
   // Has never been executed.
@@ -957,24 +986,26 @@ enum InlineCacheState {
   GENERIC,
 };
 
+inline size_t hash_value(InlineCacheState mode) { return bit_cast<int>(mode); }
+
 // Printing support.
 inline const char* InlineCacheState2String(InlineCacheState state) {
   switch (state) {
-    case NO_FEEDBACK:
+    case InlineCacheState::NO_FEEDBACK:
       return "NOFEEDBACK";
-    case UNINITIALIZED:
+    case InlineCacheState::UNINITIALIZED:
       return "UNINITIALIZED";
-    case MONOMORPHIC:
+    case InlineCacheState::MONOMORPHIC:
       return "MONOMORPHIC";
-    case RECOMPUTE_HANDLER:
+    case InlineCacheState::RECOMPUTE_HANDLER:
       return "RECOMPUTE_HANDLER";
-    case POLYMORPHIC:
+    case InlineCacheState::POLYMORPHIC:
       return "POLYMORPHIC";
-    case MEGAMORPHIC:
+    case InlineCacheState::MEGAMORPHIC:
       return "MEGAMORPHIC";
-    case MEGADOM:
+    case InlineCacheState::MEGADOM:
       return "MEGADOM";
-    case GENERIC:
+    case InlineCacheState::GENERIC:
       return "GENERIC";
   }
   UNREACHABLE();
@@ -1613,7 +1644,7 @@ inline std::ostream& operator<<(std::ostream& os,
 
 using FileAndLine = std::pair<const char*, int>;
 
-enum OptimizationMarker : int32_t {
+enum class OptimizationMarker : int32_t {
   // These values are set so that it is easy to check if there is a marker where
   // some processing needs to be done.
   kNone = 0b000,
@@ -1626,8 +1657,11 @@ enum OptimizationMarker : int32_t {
 // For kNone or kInOptimizationQueue we don't need any special processing.
 // To check both cases using a single mask, we expect the kNone to be 0 and
 // kInOptimizationQueue to be 1 so that we can mask off the lsb for checking.
-STATIC_ASSERT(kNone == 0b000 && kInOptimizationQueue == 0b001);
-STATIC_ASSERT(kLastOptimizationMarker <= 0b111);
+STATIC_ASSERT(static_cast<int>(OptimizationMarker::kNone) == 0b000 &&
+              static_cast<int>(OptimizationMarker::kInOptimizationQueue) ==
+                  0b001);
+STATIC_ASSERT(static_cast<int>(OptimizationMarker::kLastOptimizationMarker) <=
+              0b111);
 static constexpr uint32_t kNoneOrInOptimizationQueueMask = 0b110;
 
 inline bool IsInOptimizationQueueMarker(OptimizationMarker marker) {
@@ -1760,7 +1794,7 @@ inline bool IsGrowStoreMode(KeyedAccessStoreMode store_mode) {
   return store_mode == STORE_AND_GROW_HANDLE_COW;
 }
 
-enum IcCheckType { ELEMENT, PROPERTY };
+enum class IcCheckType { kElement, kProperty };
 
 // Helper stubs can be called in different ways depending on where the target
 // code is located and how the call sequence is expected to look like:
@@ -1880,13 +1914,13 @@ enum PropertiesEnumerationMode {
   kPropertyAdditionOrder,
 };
 
-enum class StringInternalizationStrategy {
-  // The string must be internalized by first copying.
+enum class StringTransitionStrategy {
+  // The string must be transitioned to a new representation by first copying.
   kCopy,
-  // The string can be internalized in-place by changing its map.
+  // The string can be transitioned in-place by changing its map.
   kInPlace,
-  // The string is already internalized.
-  kAlreadyInternalized
+  // The string is already transitioned to the desired representation.
+  kAlreadyTransitioned
 };
 
 }  // namespace internal
