@@ -797,7 +797,7 @@ PropertyAccessInfo AccessInfoFactory::ComputePropertyAccessInfo(
         // Don't bother optimizing stores to read-only properties.
         if (details.IsReadOnly()) return Invalid();
 
-        if (details.kind() == kData && holder.has_value()) {
+        if (details.kind() == PropertyKind::kData && holder.has_value()) {
           // This is a store to a property not found on the receiver but on a
           // prototype. According to ES6 section 9.1.9 [[Set]], we need to
           // create a new data property on the receiver. We can still optimize
@@ -841,17 +841,17 @@ PropertyAccessInfo AccessInfoFactory::ComputePropertyAccessInfo(
         return Invalid();
       }
       if (details.location() == PropertyLocation::kField) {
-        if (details.kind() == kData) {
+        if (details.kind() == PropertyKind::kData) {
           return ComputeDataFieldAccessInfo(receiver_map, map, holder, index,
                                             access_mode);
         } else {
-          DCHECK_EQ(kAccessor, details.kind());
+          DCHECK_EQ(PropertyKind::kAccessor, details.kind());
           // TODO(turbofan): Add support for general accessors?
           return Invalid();
         }
       } else {
         DCHECK_EQ(PropertyLocation::kDescriptor, details.location());
-        DCHECK_EQ(kAccessor, details.kind());
+        DCHECK_EQ(PropertyKind::kAccessor, details.kind());
         return ComputeAccessorDescriptorAccessInfo(receiver_map, name, map,
                                                    holder, index, access_mode);
       }
@@ -1124,9 +1124,10 @@ PropertyAccessInfo AccessInfoFactory::LookupTransition(
     MapRef map, NameRef name, base::Optional<JSObjectRef> holder,
     PropertyAttributes attrs) const {
   // Check if the {map} has a data transition with the given {name}.
-  Map transition = TransitionsAccessor(isolate(), map.object(),
-                                       broker()->is_concurrent_inlining())
-                       .SearchTransition(*name.object(), kData, attrs);
+  Map transition =
+      TransitionsAccessor(isolate(), map.object(),
+                          broker()->is_concurrent_inlining())
+          .SearchTransition(*name.object(), PropertyKind::kData, attrs);
   if (transition.is_null()) return Invalid();
 
   base::Optional<MapRef> maybe_transition_map =
