@@ -14,6 +14,7 @@
 #include "src/base/memory.h"
 #include "src/base/strings.h"
 #include "src/base/v8-fallthrough.h"
+#include "src/codegen/x64/fma-instr.h"
 #include "src/codegen/x64/register-x64.h"
 #include "src/codegen/x64/sse-instr.h"
 #include "src/common/globals.h"
@@ -893,96 +894,6 @@ int DisassemblerX64::AVXInstruction(byte* data) {
         AppendToBuffer("vbroadcastss %s,", NameOfAVXRegister(regop));
         current += PrintRightXMMOperand(current);
         break;
-      case 0x98:
-        AppendToBuffer("vfmadd132p%c %s,%s,", float_size_code(),
-                       NameOfXMMRegister(regop), NameOfXMMRegister(vvvv));
-        current += PrintRightXMMOperand(current);
-        break;
-      case 0x99:
-        AppendToBuffer("vfmadd132s%c %s,%s,", float_size_code(),
-                       NameOfAVXRegister(regop), NameOfAVXRegister(vvvv));
-        current += PrintRightAVXOperand(current);
-        break;
-      case 0xA8:
-        AppendToBuffer("vfmadd213p%c %s,%s,", float_size_code(),
-                       NameOfXMMRegister(regop), NameOfXMMRegister(vvvv));
-        current += PrintRightXMMOperand(current);
-        break;
-      case 0xA9:
-        AppendToBuffer("vfmadd213s%c %s,%s,", float_size_code(),
-                       NameOfAVXRegister(regop), NameOfAVXRegister(vvvv));
-        current += PrintRightAVXOperand(current);
-        break;
-      case 0xB8:
-        AppendToBuffer("vfmadd231p%c %s,%s,", float_size_code(),
-                       NameOfAVXRegister(regop), NameOfAVXRegister(vvvv));
-        current += PrintRightAVXOperand(current);
-        break;
-      case 0xB9:
-        AppendToBuffer("vfmadd231s%c %s,%s,", float_size_code(),
-                       NameOfAVXRegister(regop), NameOfAVXRegister(vvvv));
-        current += PrintRightAVXOperand(current);
-        break;
-      case 0x9B:
-        AppendToBuffer("vfmsub132s%c %s,%s,", float_size_code(),
-                       NameOfAVXRegister(regop), NameOfAVXRegister(vvvv));
-        current += PrintRightAVXOperand(current);
-        break;
-      case 0x9C:
-        AppendToBuffer("vfnmadd132p%c %s,%s,", float_size_code(),
-                       NameOfXMMRegister(regop), NameOfXMMRegister(vvvv));
-        current += PrintRightXMMOperand(current);
-        break;
-      case 0xAB:
-        AppendToBuffer("vfmsub213s%c %s,%s,", float_size_code(),
-                       NameOfAVXRegister(regop), NameOfAVXRegister(vvvv));
-        current += PrintRightAVXOperand(current);
-        break;
-      case 0xAC:
-        AppendToBuffer("vfnmadd213p%c %s,%s,", float_size_code(),
-                       NameOfXMMRegister(regop), NameOfXMMRegister(vvvv));
-        current += PrintRightXMMOperand(current);
-        break;
-      case 0xBB:
-        AppendToBuffer("vfmsub231s%c %s,%s,", float_size_code(),
-                       NameOfAVXRegister(regop), NameOfAVXRegister(vvvv));
-        current += PrintRightAVXOperand(current);
-        break;
-      case 0xBC:
-        AppendToBuffer("vfnmadd231p%c %s,%s,", float_size_code(),
-                       NameOfAVXRegister(regop), NameOfAVXRegister(vvvv));
-        current += PrintRightAVXOperand(current);
-        break;
-      case 0x9D:
-        AppendToBuffer("vfnmadd132s%c %s,%s,", float_size_code(),
-                       NameOfAVXRegister(regop), NameOfAVXRegister(vvvv));
-        current += PrintRightAVXOperand(current);
-        break;
-      case 0xAD:
-        AppendToBuffer("vfnmadd213s%c %s,%s,", float_size_code(),
-                       NameOfAVXRegister(regop), NameOfAVXRegister(vvvv));
-        current += PrintRightAVXOperand(current);
-        break;
-      case 0xBD:
-        AppendToBuffer("vfnmadd231s%c %s,%s,", float_size_code(),
-                       NameOfAVXRegister(regop), NameOfAVXRegister(vvvv));
-        current += PrintRightAVXOperand(current);
-        break;
-      case 0x9F:
-        AppendToBuffer("vfnmsub132s%c %s,%s,", float_size_code(),
-                       NameOfAVXRegister(regop), NameOfAVXRegister(vvvv));
-        current += PrintRightAVXOperand(current);
-        break;
-      case 0xAF:
-        AppendToBuffer("vfnmsub213s%c %s,%s,", float_size_code(),
-                       NameOfAVXRegister(regop), NameOfAVXRegister(vvvv));
-        current += PrintRightAVXOperand(current);
-        break;
-      case 0xBF:
-        AppendToBuffer("vfnmsub231s%c %s,%s,", float_size_code(),
-                       NameOfAVXRegister(regop), NameOfAVXRegister(vvvv));
-        current += PrintRightAVXOperand(current);
-        break;
       case 0xF7:
         AppendToBuffer("shlx%c %s,", operand_size_code(),
                        NameOfCPURegister(regop));
@@ -1022,8 +933,35 @@ int DisassemblerX64::AVXInstruction(byte* data) {
         AVX2_BROADCAST_LIST(DISASSEMBLE_AVX2_BROADCAST)
 #undef DISASSEMBLE_AVX2_BROADCAST
 
-      default:
-        UnimplementedInstruction();
+      default: {
+#define DECLARE_FMA_DISASM(instruction, _1, _2, _3, _4, _5, code)    \
+  case 0x##code: {                                                   \
+    AppendToBuffer(#instruction " %s,%s,", NameOfAVXRegister(regop), \
+                   NameOfAVXRegister(vvvv));                         \
+    current += PrintRightAVXOperand(current);                        \
+    break;                                                           \
+  }
+        // Handle all the fma instructions here in the default branch since they
+        // have the same opcodes but differ by rex_w.
+        if (rex_w()) {
+          switch (opcode) {
+            FMA_SS_INSTRUCTION_LIST(DECLARE_FMA_DISASM)
+            FMA_PS_INSTRUCTION_LIST(DECLARE_FMA_DISASM)
+            default: {
+              UnimplementedInstruction();
+            }
+          }
+        } else {
+          switch (opcode) {
+            FMA_SD_INSTRUCTION_LIST(DECLARE_FMA_DISASM)
+            FMA_PD_INSTRUCTION_LIST(DECLARE_FMA_DISASM)
+            default: {
+              UnimplementedInstruction();
+            }
+          }
+        }
+#undef DECLARE_FMA_DISASM
+      }
     }
   } else if (vex_66() && vex_0f3a()) {
     int mod, regop, rm, vvvv = vex_vreg();
@@ -1956,7 +1894,6 @@ int DisassemblerX64::TwoByteOpcodeInstruction(byte* data) {
     } else if (opcode == 0xD7) {
       current += PrintOperands("pmovmskb", OPER_XMMREG_OP_ORDER, current);
     } else {
-      const char* mnemonic;
 #define SSE2_CASE(instruction, notUsed1, notUsed2, opcode) \
   case 0x##opcode:                                         \
     mnemonic = "" #instruction;                            \
