@@ -158,6 +158,8 @@ bool StringShape::IsIndirect() const {
   return (type_ & kIsIndirectStringMask) == kIsIndirectStringTag;
 }
 
+bool StringShape::IsDirect() const { return !IsIndirect(); }
+
 bool StringShape::IsExternal() const {
   return (type_ & kStringRepresentationMask) == kExternalStringTag;
 }
@@ -170,6 +172,31 @@ bool StringShape::IsUncachedExternal() const {
   return (type_ & kUncachedExternalStringMask) == kUncachedExternalStringTag;
 }
 
+bool StringShape::IsShared() const {
+  // TODO(v8:12007): Set is_shared to true on internalized string when
+  // FLAG_shared_string_table is removed.
+  return (type_ & kSharedStringMask) == kSharedStringTag ||
+         (FLAG_shared_string_table && IsInternalized());
+}
+
+bool StringShape::CanMigrateInParallel() const {
+  switch (representation_encoding_and_shared_tag()) {
+    case kSeqOneByteStringTag | kSharedStringTag:
+    case kSeqTwoByteStringTag | kSharedStringTag:
+      // Shared SeqStrings can migrate to ThinStrings.
+      return true;
+    case kThinStringTag | kOneByteStringTag | kSharedStringTag:
+    case kThinStringTag | kTwoByteStringTag | kSharedStringTag:
+      // Shared ThinStrings do not migrate.
+      return false;
+    default:
+      // If you crashed here, you probably added a new shared string
+      // type. Explicitly handle all shared string cases above.
+      DCHECK(!IsShared());
+      return false;
+  }
+}
+
 StringRepresentationTag StringShape::representation_tag() const {
   uint32_t tag = (type_ & kStringRepresentationMask);
   return static_cast<StringRepresentationTag>(tag);
@@ -179,45 +206,49 @@ uint32_t StringShape::encoding_tag() const {
   return type_ & kStringEncodingMask;
 }
 
-uint32_t StringShape::full_representation_tag() const {
-  return (type_ & (kStringRepresentationMask | kStringEncodingMask));
+uint32_t StringShape::representation_and_encoding_tag() const {
+  return (type_ & (kStringRepresentationAndEncodingMask));
 }
 
-STATIC_ASSERT((kStringRepresentationMask | kStringEncodingMask) ==
-              Internals::kFullStringRepresentationMask);
+uint32_t StringShape::representation_encoding_and_shared_tag() const {
+  return (type_ & (kStringRepresentationEncodingAndSharedMask));
+}
+
+STATIC_ASSERT((kStringRepresentationAndEncodingMask) ==
+              Internals::kStringRepresentationAndEncodingMask);
 
 STATIC_ASSERT(static_cast<uint32_t>(kStringEncodingMask) ==
               Internals::kStringEncodingMask);
 
 bool StringShape::IsSequentialOneByte() const {
-  return full_representation_tag() == (kSeqStringTag | kOneByteStringTag);
+  return representation_and_encoding_tag() == kSeqOneByteStringTag;
 }
 
 bool StringShape::IsSequentialTwoByte() const {
-  return full_representation_tag() == (kSeqStringTag | kTwoByteStringTag);
+  return representation_and_encoding_tag() == kSeqTwoByteStringTag;
 }
 
 bool StringShape::IsExternalOneByte() const {
-  return full_representation_tag() == (kExternalStringTag | kOneByteStringTag);
+  return representation_and_encoding_tag() == kExternalOneByteStringTag;
 }
 
-STATIC_ASSERT((kExternalStringTag | kOneByteStringTag) ==
+STATIC_ASSERT(kExternalOneByteStringTag ==
               Internals::kExternalOneByteRepresentationTag);
 
 STATIC_ASSERT(v8::String::ONE_BYTE_ENCODING == kOneByteStringTag);
 
 bool StringShape::IsExternalTwoByte() const {
-  return full_representation_tag() == (kExternalStringTag | kTwoByteStringTag);
+  return representation_and_encoding_tag() == kExternalTwoByteStringTag;
 }
 
-STATIC_ASSERT((kExternalStringTag | kTwoByteStringTag) ==
+STATIC_ASSERT(kExternalTwoByteStringTag ==
               Internals::kExternalTwoByteRepresentationTag);
 
 STATIC_ASSERT(v8::String::TWO_BYTE_ENCODING == kTwoByteStringTag);
 
 template <typename TDispatcher, typename TResult, typename... TArgs>
 inline TResult StringShape::DispatchToSpecificTypeWithoutCast(TArgs&&... args) {
-  switch (full_representation_tag()) {
+  switch (representation_and_encoding_tag()) {
     case kSeqStringTag | kOneByteStringTag:
       return TDispatcher::HandleSeqOneByteString(std::forward<TArgs>(args)...);
     case kSeqStringTag | kTwoByteStringTag:
@@ -499,23 +530,23 @@ bool String::IsEqualToImpl(
   const Char* data = str.data();
   while (true) {
     int32_t type = string.map(cage_base).instance_type();
-    switch (type & (kStringRepresentationMask | kStringEncodingMask)) {
-      case kSeqStringTag | kOneByteStringTag:
+    switch (type & kStringRepresentationAndEncodingMask) {
+      case kSeqOneByteStringTag:
         return CompareCharsEqual(
             SeqOneByteString::cast(string).GetChars(no_gc, access_guard) +
                 slice_offset,
             data, len);
-      case kSeqStringTag | kTwoByteStringTag:
+      case kSeqTwoByteStringTag:
         return CompareCharsEqual(
             SeqTwoByteString::cast(string).GetChars(no_gc, access_guard) +
                 slice_offset,
             data, len);
-      case kExternalStringTag | kOneByteStringTag:
+      case kExternalOneByteStringTag:
         return CompareCharsEqual(
             ExternalOneByteString::cast(string).GetChars(cage_base) +
                 slice_offset,
             data, len);
-      case kExternalStringTag | kTwoByteStringTag:
+      case kExternalTwoByteStringTag:
         return CompareCharsEqual(
             ExternalTwoByteString::cast(string).GetChars(cage_base) +
                 slice_offset,
@@ -604,20 +635,31 @@ const Char* String::GetChars(
 
 Handle<String> String::Flatten(Isolate* isolate, Handle<String> string,
                                AllocationType allocation) {
-  if (string->IsConsString()) {
-    DCHECK(!string->InSharedHeap());
-    Handle<ConsString> cons = Handle<ConsString>::cast(string);
-    if (cons->IsFlat()) {
-      string = handle(cons->first(), isolate);
-    } else {
-      return SlowFlatten(isolate, cons, allocation);
+  DisallowGarbageCollection no_gc;  // Unhandlified code.
+  PtrComprCageBase cage_base(isolate);
+  String s = *string;
+  StringShape shape(s, cage_base);
+
+  // Shortcut already-flat strings.
+  if (V8_LIKELY(shape.IsDirect())) return string;
+
+  if (shape.IsCons()) {
+    DCHECK(!s.InSharedHeap());
+    ConsString cons = ConsString::cast(s);
+    if (!cons.IsFlat(isolate)) {
+      AllowGarbageCollection yes_gc;
+      return SlowFlatten(isolate, handle(cons, isolate), allocation);
     }
+    s = cons.first(cage_base);
+    shape = StringShape(s, cage_base);
   }
-  if (string->IsThinString()) {
-    string = handle(Handle<ThinString>::cast(string)->actual(), isolate);
-    DCHECK(!string->IsConsString());
+
+  if (shape.IsThin()) {
+    s = ThinString::cast(s).actual(cage_base);
+    DCHECK(!s.IsConsString());
   }
-  return string;
+
+  return handle(s, isolate);
 }
 
 Handle<String> String::Flatten(LocalIsolate* isolate, Handle<String> string,
@@ -627,6 +669,80 @@ Handle<String> String::Flatten(LocalIsolate* isolate, Handle<String> string,
   return string;
 }
 
+// static
+base::Optional<String::FlatContent> String::TryGetFlatContentFromDirectString(
+    PtrComprCageBase cage_base, const DisallowGarbageCollection& no_gc,
+    String string, int offset, int length,
+    const SharedStringAccessGuardIfNeeded& access_guard) {
+  DCHECK_GE(offset, 0);
+  DCHECK_GE(length, 0);
+  DCHECK_LE(offset + length, string.length());
+  switch (StringShape{string, cage_base}.representation_and_encoding_tag()) {
+    case kSeqOneByteStringTag:
+      return FlatContent(
+          SeqOneByteString::cast(string).GetChars(no_gc, access_guard) + offset,
+          length, no_gc);
+    case kSeqTwoByteStringTag:
+      return FlatContent(
+          SeqTwoByteString::cast(string).GetChars(no_gc, access_guard) + offset,
+          length, no_gc);
+    case kExternalOneByteStringTag:
+      return FlatContent(
+          ExternalOneByteString::cast(string).GetChars(cage_base) + offset,
+          length, no_gc);
+    case kExternalTwoByteStringTag:
+      return FlatContent(
+          ExternalTwoByteString::cast(string).GetChars(cage_base) + offset,
+          length, no_gc);
+    default:
+      return {};
+  }
+  UNREACHABLE();
+}
+
+String::FlatContent String::GetFlatContent(
+    const DisallowGarbageCollection& no_gc) {
+#if DEBUG
+  // Check that this method is called only from the main thread.
+  {
+    Isolate* isolate;
+    // We don't have to check read only strings as those won't move.
+    DCHECK_IMPLIES(GetIsolateFromHeapObject(*this, &isolate),
+                   ThreadId::Current() == isolate->thread_id());
+  }
+#endif
+
+  return GetFlatContent(no_gc, SharedStringAccessGuardIfNeeded::NotNeeded());
+}
+
+String::FlatContent String::GetFlatContent(
+    const DisallowGarbageCollection& no_gc,
+    const SharedStringAccessGuardIfNeeded& access_guard) {
+  PtrComprCageBase cage_base = GetPtrComprCageBase(*this);
+  base::Optional<FlatContent> flat_content = TryGetFlatContentFromDirectString(
+      cage_base, no_gc, *this, 0, length(), access_guard);
+  if (flat_content.has_value()) return flat_content.value();
+  return SlowGetFlatContent(no_gc, access_guard);
+}
+
+Handle<String> String::Share(Isolate* isolate, Handle<String> string) {
+  DCHECK(FLAG_shared_string_table);
+  MaybeHandle<Map> new_map;
+  switch (
+      isolate->factory()->ComputeSharingStrategyForString(string, &new_map)) {
+    case StringTransitionStrategy::kCopy:
+      return SlowShare(isolate, string);
+    case StringTransitionStrategy::kInPlace:
+      // A relaxed write is sufficient here, because at this point the string
+      // has not yet escaped the current thread.
+      DCHECK(string->InSharedHeap());
+      string->set_map_no_write_barrier(*new_map.ToHandleChecked());
+      return string;
+    case StringTransitionStrategy::kAlreadyTransitioned:
+      return string;
+  }
+}
+
 uint16_t String::Get(int index) const {
   DCHECK(!SharedStringAccessGuardIfNeeded::IsNeeded(*this));
   return GetImpl(index, GetPtrComprCageBase(*this),
@@ -680,7 +796,7 @@ void String::Set(int index, uint16_t value) {
   DCHECK(index >= 0 && index < length());
   DCHECK(StringShape(*this).IsSequential());
 
-  return this->IsOneByteRepresentation()
+  return IsOneByteRepresentation()
              ? SeqOneByteString::cast(*this).SeqOneByteStringSet(index, value)
              : SeqTwoByteString::cast(*this).SeqTwoByteStringSet(index, value);
 }
@@ -689,13 +805,21 @@ bool String::IsFlat() const { return IsFlat(GetPtrComprCageBase(*this)); }
 
 bool String::IsFlat(PtrComprCageBase cage_base) const {
   if (!StringShape(*this, cage_base).IsCons()) return true;
-  return ConsString::cast(*this).second(cage_base).length() == 0;
+  return ConsString::cast(*this).IsFlat(cage_base);
+}
+
+bool String::IsShared() const { return IsShared(GetPtrComprCageBase(*this)); }
+
+bool String::IsShared(PtrComprCageBase cage_base) const {
+  const bool result = StringShape(*this, cage_base).IsShared();
+  DCHECK_IMPLIES(result, InSharedHeap());
+  return result;
 }
 
 String String::GetUnderlying() const {
   // Giving direct access to underlying string only makes sense if the
   // wrapping string is already flattened.
-  DCHECK(this->IsFlat());
+  DCHECK(IsFlat());
   DCHECK(StringShape(*this).IsIndirect());
   STATIC_ASSERT(static_cast<int>(ConsString::kFirstOffset) ==
                 static_cast<int>(SlicedString::kParentOffset));
@@ -723,30 +847,31 @@ ConsString String::VisitFlat(
   DCHECK(offset <= length);
   PtrComprCageBase cage_base = GetPtrComprCageBase(string);
   while (true) {
-    int32_t tag = StringShape(string, cage_base).full_representation_tag();
+    int32_t tag =
+        StringShape(string, cage_base).representation_and_encoding_tag();
     switch (tag) {
-      case kSeqStringTag | kOneByteStringTag:
+      case kSeqOneByteStringTag:
         visitor->VisitOneByteString(
             SeqOneByteString::cast(string).GetChars(no_gc, access_guard) +
                 slice_offset,
             length - offset);
         return ConsString();
 
-      case kSeqStringTag | kTwoByteStringTag:
+      case kSeqTwoByteStringTag:
         visitor->VisitTwoByteString(
             SeqTwoByteString::cast(string).GetChars(no_gc, access_guard) +
                 slice_offset,
             length - offset);
         return ConsString();
 
-      case kExternalStringTag | kOneByteStringTag:
+      case kExternalOneByteStringTag:
         visitor->VisitOneByteString(
             ExternalOneByteString::cast(string).GetChars(cage_base) +
                 slice_offset,
             length - offset);
         return ConsString();
 
-      case kExternalStringTag | kTwoByteStringTag:
+      case kExternalTwoByteStringTag:
         visitor->VisitTwoByteString(
             ExternalTwoByteString::cast(string).GetChars(cage_base) +
                 slice_offset,
@@ -807,10 +932,22 @@ uint8_t SeqOneByteString::Get(
 }
 
 void SeqOneByteString::SeqOneByteStringSet(int index, uint16_t value) {
-  DCHECK(index >= 0 && index < length() && value <= kMaxOneByteCharCode);
+  DCHECK_GE(index, 0);
+  DCHECK_LT(index, length());
+  DCHECK_LE(value, kMaxOneByteCharCode);
   WriteField<byte>(kHeaderSize + index * kCharSize, static_cast<byte>(value));
 }
 
+void SeqOneByteString::SeqOneByteStringSetChars(int index,
+                                                const uint8_t* string,
+                                                int string_length) {
+  DCHECK_LE(0, index);
+  DCHECK_LT(index + string_length, length());
+  void* address =
+      reinterpret_cast<void*>(field_address(kHeaderSize + index * kCharSize));
+  memcpy(address, string, string_length);
+}
+
 Address SeqOneByteString::GetCharsAddress() const {
   return field_address(kHeaderSize);
 }
@@ -871,19 +1008,34 @@ inline int SeqTwoByteString::AllocatedSize() {
   return SizeFor(length(kAcquireLoad));
 }
 
+// static
+bool SeqOneByteString::IsCompatibleMap(Map map, ReadOnlyRoots roots) {
+  return map == roots.one_byte_string_map() ||
+         map == roots.shared_one_byte_string_map();
+}
+
+// static
+bool SeqTwoByteString::IsCompatibleMap(Map map, ReadOnlyRoots roots) {
+  return map == roots.string_map() || map == roots.shared_string_map();
+}
+
 void SlicedString::set_parent(String parent, WriteBarrierMode mode) {
   DCHECK(parent.IsSeqString() || parent.IsExternalString());
   TorqueGeneratedSlicedString<SlicedString, Super>::set_parent(parent, mode);
 }
 
-Object ConsString::unchecked_first() {
+Object ConsString::unchecked_first() const {
   return TaggedField<Object, kFirstOffset>::load(*this);
 }
 
-Object ConsString::unchecked_second() {
+Object ConsString::unchecked_second() const {
   return RELAXED_READ_FIELD(*this, kSecondOffset);
 }
 
+bool ConsString::IsFlat(PtrComprCageBase cage_base) const {
+  return second(cage_base).length() == 0;
+}
+
 DEF_GETTER(ThinString, unchecked_actual, HeapObject) {
   return TaggedField<HeapObject, kActualOffset>::load(cage_base, *this);
 }
@@ -1272,6 +1424,8 @@ bool String::IsInPlaceInternalizable(InstanceType instance_type) {
   switch (instance_type) {
     case STRING_TYPE:
     case ONE_BYTE_STRING_TYPE:
+    case SHARED_STRING_TYPE:
+    case SHARED_ONE_BYTE_STRING_TYPE:
     case EXTERNAL_STRING_TYPE:
     case EXTERNAL_ONE_BYTE_STRING_TYPE:
       return true;
