@@ -563,6 +563,8 @@ class WasmCodeAllocator {
   // Hold the {NativeModule}'s {allocation_mutex_} when calling this method.
   size_t GetNumCodeSpaces() const;
 
+  Counters* counters() const { return async_counters_.get(); }
+
  private:
   // Sentinel value to be used for {AllocateForCodeInRegion} for specifying no
   // restriction on the region to allocate in.
@@ -839,9 +841,9 @@ class V8_EXPORT_PRIVATE NativeModule final {
   // Get or create the debug info for this NativeModule.
   DebugInfo* GetDebugInfo();
 
-  uint32_t* num_liftoff_function_calls_array() {
-    return num_liftoff_function_calls_.get();
-  }
+  uint32_t* tiering_budget_array() { return tiering_budgets_.get(); }
+
+  Counters* counters() const { return code_allocator_.counters(); }
 
  private:
   friend class WasmCode;
@@ -944,7 +946,7 @@ class V8_EXPORT_PRIVATE NativeModule final {
   std::unique_ptr<WasmImportWrapperCache> import_wrapper_cache_;
 
   // Array to handle number of function calls.
-  std::unique_ptr<uint32_t[]> num_liftoff_function_calls_;
+  std::unique_ptr<uint32_t[]> tiering_budgets_;
 
   // This mutex protects concurrent calls to {AddCode} and friends.
   // TODO(dlehmann): Revert this to a regular {Mutex} again.
@@ -1043,9 +1045,14 @@ class V8_EXPORT_PRIVATE WasmCodeManager final {
   // lock when calling this method.
   void SetThreadWritable(bool writable);
 
-  // Returns true if there is PKU support, false otherwise.
+  // Returns true if there is hardware support for PKU. Use
+  // {MemoryProtectionKeysEnabled} to also check if PKU usage is enabled via
+  // flags.
   bool HasMemoryProtectionKeySupport() const;
 
+  // Returns true if PKU should be used.
+  bool MemoryProtectionKeysEnabled() const;
+
   // Returns {true} if the memory protection key is write-enabled for the
   // current thread.
   // Can only be called if {HasMemoryProtectionKeySupport()} is {true}.
