@@ -48,10 +48,12 @@ TQ_OBJECT_CONSTRUCTORS_IMPL(WasmTableObject)
 TQ_OBJECT_CONSTRUCTORS_IMPL(AsmWasmData)
 TQ_OBJECT_CONSTRUCTORS_IMPL(WasmFunctionData)
 TQ_OBJECT_CONSTRUCTORS_IMPL(WasmApiFunctionRef)
+TQ_OBJECT_CONSTRUCTORS_IMPL(WasmInternalFunction)
 TQ_OBJECT_CONSTRUCTORS_IMPL(WasmTypeInfo)
 TQ_OBJECT_CONSTRUCTORS_IMPL(WasmStruct)
 TQ_OBJECT_CONSTRUCTORS_IMPL(WasmArray)
 TQ_OBJECT_CONSTRUCTORS_IMPL(WasmContinuationObject)
+TQ_OBJECT_CONSTRUCTORS_IMPL(WasmSuspenderObject)
 
 CAST_ACCESSOR(WasmInstanceObject)
 
@@ -176,13 +178,12 @@ void WasmGlobalObject::SetExternRef(Handle<Object> value) {
 
 bool WasmGlobalObject::SetFuncRef(Isolate* isolate, Handle<Object> value) {
   DCHECK_EQ(type(), wasm::kWasmFuncRef);
-  if (!value->IsNull(isolate) &&
-      !WasmExternalFunction::IsWasmExternalFunction(*value) &&
-      !WasmCapiFunction::IsWasmCapiFunction(*value)) {
-    return false;
+  if (value->IsNull() ||
+      WasmInternalFunction::FromExternal(value, isolate).ToHandle(&value)) {
+    tagged_buffer().set(offset(), *value);
+    return true;
   }
-  tagged_buffer().set(offset(), *value);
-  return true;
+  return false;
 }
 
 // WasmInstanceObject
@@ -224,10 +225,8 @@ PRIMITIVE_ACCESSORS(WasmInstanceObject, dropped_elem_segments, byte*,
                     kDroppedElemSegmentsOffset)
 PRIMITIVE_ACCESSORS(WasmInstanceObject, hook_on_function_call_address, Address,
                     kHookOnFunctionCallAddressOffset)
-PRIMITIVE_ACCESSORS(WasmInstanceObject, num_liftoff_function_calls_array,
-                    uint32_t*, kNumLiftoffFunctionCallsArrayOffset)
-ACCESSORS(WasmInstanceObject, active_continuation, WasmContinuationObject,
-          kActiveContinuationOffset)
+PRIMITIVE_ACCESSORS(WasmInstanceObject, tiering_budget_array, uint32_t*,
+                    kTieringBudgetArrayOffset)
 PRIMITIVE_ACCESSORS(WasmInstanceObject, break_on_entry, uint8_t,
                     kBreakOnEntryOffset)
 
@@ -253,8 +252,8 @@ OPTIONAL_ACCESSORS(WasmInstanceObject, indirect_function_table_refs, FixedArray,
 OPTIONAL_ACCESSORS(WasmInstanceObject, managed_native_allocations, Foreign,
                    kManagedNativeAllocationsOffset)
 OPTIONAL_ACCESSORS(WasmInstanceObject, tags_table, FixedArray, kTagsTableOffset)
-OPTIONAL_ACCESSORS(WasmInstanceObject, wasm_external_functions, FixedArray,
-                   kWasmExternalFunctionsOffset)
+OPTIONAL_ACCESSORS(WasmInstanceObject, wasm_internal_functions, FixedArray,
+                   kWasmInternalFunctionsOffset)
 ACCESSORS(WasmInstanceObject, managed_object_maps, FixedArray,
           kManagedObjectMapsOffset)
 ACCESSORS(WasmInstanceObject, feedback_vectors, FixedArray,
@@ -268,32 +267,6 @@ void WasmInstanceObject::clear_padding() {
   }
 }
 
-IndirectFunctionTableEntry::IndirectFunctionTableEntry(
-    Handle<WasmInstanceObject> instance, int table_index, int entry_index)
-    : instance_(table_index == 0 ? instance
-                                 : Handle<WasmInstanceObject>::null()),
-      table_(table_index != 0
-                 ? handle(WasmIndirectFunctionTable::cast(
-                              instance->indirect_function_tables().get(
-                                  table_index)),
-                          instance->GetIsolate())
-                 : Handle<WasmIndirectFunctionTable>::null()),
-      index_(entry_index) {
-  DCHECK_GE(entry_index, 0);
-  DCHECK_LT(entry_index, table_index == 0
-                             ? instance->indirect_function_table_size()
-                             : table_->size());
-}
-
-IndirectFunctionTableEntry::IndirectFunctionTableEntry(
-    Handle<WasmIndirectFunctionTable> table, int entry_index)
-    : instance_(Handle<WasmInstanceObject>::null()),
-      table_(table),
-      index_(entry_index) {
-  DCHECK_GE(entry_index, 0);
-  DCHECK_LT(entry_index, table_->size());
-}
-
 ImportedFunctionEntry::ImportedFunctionEntry(
     Handle<WasmInstanceObject> instance, int index)
     : instance_(instance), index_(index) {
@@ -312,7 +285,7 @@ WasmExportedFunction::WasmExportedFunction(Address ptr) : JSFunction(ptr) {
 CAST_ACCESSOR(WasmExportedFunction)
 
 // WasmFunctionData
-ACCESSORS(WasmFunctionData, ref, Object, kRefOffset)
+ACCESSORS(WasmFunctionData, internal, WasmInternalFunction, kInternalOffset)
 
 DEF_GETTER(WasmFunctionData, wrapper_code, Code) {
   return FromCodeT(TorqueGeneratedClass::wrapper_code(cage_base));
@@ -333,15 +306,15 @@ CAST_ACCESSOR(WasmJSFunction)
 
 // WasmJSFunctionData
 TQ_OBJECT_CONSTRUCTORS_IMPL(WasmJSFunctionData)
-ACCESSORS(WasmJSFunctionData, raw_wasm_to_js_wrapper_code, CodeT,
-          kWasmToJsWrapperCodeOffset)
 
-DEF_GETTER(WasmJSFunctionData, wasm_to_js_wrapper_code, Code) {
-  return FromCodeT(raw_wasm_to_js_wrapper_code(cage_base));
+// WasmInternalFunction
+ACCESSORS(WasmInternalFunction, raw_code, CodeT, kCodeOffset)
+
+DEF_GETTER(WasmInternalFunction, code, Code) {
+  return FromCodeT(raw_code(cage_base));
 }
-void WasmJSFunctionData::set_wasm_to_js_wrapper_code(Code code,
-                                                     WriteBarrierMode mode) {
-  set_raw_wasm_to_js_wrapper_code(ToCodeT(code), mode);
+void WasmInternalFunction::set_code(Code code, WriteBarrierMode mode) {
+  set_raw_code(ToCodeT(code), mode);
 }
 
 // WasmCapiFunction
@@ -548,10 +521,10 @@ wasm::StructType* WasmStruct::type(Map map) {
 wasm::StructType* WasmStruct::GcSafeType(Map map) {
   DCHECK_EQ(WASM_STRUCT_TYPE, map.instance_type());
   HeapObject raw = HeapObject::cast(map.constructor_or_back_pointer());
-  MapWord map_word = raw.map_word(kRelaxedLoad);
-  HeapObject forwarded =
-      map_word.IsForwardingAddress() ? map_word.ToForwardingAddress() : raw;
-  Foreign foreign = Foreign::cast(forwarded);
+  // The {Foreign} might be in the middle of being moved, which is why we
+  // can't read its map for a checked cast. But we can rely on its payload
+  // being intact in the old location.
+  Foreign foreign = Foreign::unchecked_cast(raw);
   return reinterpret_cast<wasm::StructType*>(foreign.foreign_address());
 }
 
@@ -624,10 +597,10 @@ wasm::ArrayType* WasmArray::type(Map map) {
 wasm::ArrayType* WasmArray::GcSafeType(Map map) {
   DCHECK_EQ(WASM_ARRAY_TYPE, map.instance_type());
   HeapObject raw = HeapObject::cast(map.constructor_or_back_pointer());
-  MapWord map_word = raw.map_word(kRelaxedLoad);
-  HeapObject forwarded =
-      map_word.IsForwardingAddress() ? map_word.ToForwardingAddress() : raw;
-  Foreign foreign = Foreign::cast(forwarded);
+  // The {Foreign} might be in the middle of being moved, which is why we
+  // can't read its map for a checked cast. But we can rely on its payload
+  // being intact in the old location.
+  Foreign foreign = Foreign::unchecked_cast(raw);
   return reinterpret_cast<wasm::ArrayType*>(foreign.foreign_address());
 }
 
